@@ -1,29 +1,37 @@
-/* 
-   BlueZ - Bluetooth protocol stack for Linux
-   Copyright (C) 2000-2001 Qualcomm Incorporated
-   
-   Written 2000,2001 by Maxim Krasnyansky <maxk@qualcomm.com>
-   
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2 as
-   published by the Free Software Foundation;
-   
-   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
-   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
-   OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
-   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
-   NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
-   USE OR PERFORMANCE OF THIS SOFTWARE.
-   
-   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
-   TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
-*/
 /*
- * $Id: security.c,v 1.8 2003/07/24 17:34:34 holtmann Exp $
+ *
+ *  BlueZ - Bluetooth protocol stack for Linux
+ *
+ *  Copyright (C) 2000-2001  Qualcomm Incorporated
+ *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
+ *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation;
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
+ *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
+ *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
+ *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
+ *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
+ *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
+ *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
+ *  SOFTWARE IS DISCLAIMED.
+ *
+ *
+ *  $Id: security.c,v 1.12 2004/04/28 12:09:32 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -173,7 +181,7 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 	evt_link_key_notify *evt = ptr;
 	bdaddr_t *dba = &evt->bdaddr;
 	struct link_key key;
-        char sa[18];
+	char sa[18];
 
 	ba2str(sba, sa);
 	syslog(LOG_INFO, "link_key_notify (sba=%s)\n", sa);
@@ -227,10 +235,11 @@ int read_pin_code(void)
 static void call_pin_helper(int dev, struct hci_conn_info *ci)
 {
 	pin_code_reply_cp pr;
+	struct sigaction sa;
 	char addr[12], str[255], *pin, name[20];
 	FILE *pipe;
-	int len;
-	
+	int ret, len;
+
 	/* Run PIN helper in the separate process */
 	switch (fork()) {
 		case 0:
@@ -258,6 +267,11 @@ static void call_pin_helper(int dev, struct hci_conn_info *ci)
 
 	setenv("PATH", "/bin:/usr/bin:/usr/local/bin", 1);
 
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_flags = SA_NOCLDSTOP;
+	sa.sa_handler = SIG_DFL;
+	sigaction(SIGCHLD, &sa, NULL);
+
 	pipe = popen(str, "r");
 	if (!pipe) {
 		syslog(LOG_ERR, "Can't exec PIN helper. %s(%d)", strerror(errno), errno);
@@ -265,15 +279,15 @@ static void call_pin_helper(int dev, struct hci_conn_info *ci)
 	}	
 
 	pin = fgets(str, sizeof(str), pipe);
-	pclose(pipe);
+	ret = pclose(pipe);
 
 	if (!pin || strlen(pin) < 5)
-		goto reject;
+		goto nopin;
 
 	strtok(pin, "\n\r");
 
 	if (strncmp("PIN:", pin, 4))
-		goto reject;
+		goto nopin;
 
 	pin += 4;
 	len  = strlen(pin);
@@ -286,11 +300,26 @@ static void call_pin_helper(int dev, struct hci_conn_info *ci)
 			PIN_CODE_REPLY_CP_SIZE, &pr);
 	exit(0);
 
+nopin:
+	if (!pin || strncmp("ERR", pin, 3))
+		syslog(LOG_ERR, "PIN helper exited abnormally with code %d", ret);
+
 reject:
 	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY, 6, &ci->bdaddr);
 	exit(0);
 }
 
+static void request_pin(int dev, struct hci_conn_info *ci)
+{
+#ifdef ENABLE_DBUS
+	if (hcid.dbus_pin_helper) {
+		hcid_dbus_request_pin(dev, ci);
+		return;
+	}
+#endif
+	call_pin_helper(dev, ci);
+}
+
 static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 {
 	struct hci_conn_info_req *cr;
@@ -337,11 +366,11 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 			/* Outgoing connection */
 		
 			/* Let PIN helper handle that */ 
-			call_pin_helper(dev, ci);
+			request_pin(dev, ci);
 		}
 	} else {
 		/* Let PIN helper handle that */ 
-		call_pin_helper(dev, ci);
+		request_pin(dev, ci);
 	}	
 	free(cr);
 	return;
@@ -354,9 +383,10 @@ reject:
 
 gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
-	char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
+	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
 	struct hci_dev_info *di = (void *) data;
-	int len, type, dev;
+	int type, dev;
+	size_t len;
 	hci_event_hdr *eh;
 	GIOError err;
 
@@ -397,7 +427,7 @@ gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 		link_key_notify(dev, &di->bdaddr, ptr);
 		break;
 	}
-		
+
 	return TRUE;
 }
 
@@ -410,7 +440,7 @@ void start_security_manager(int hdev)
 
 	if (chan)
 		return;
-	
+
 	syslog(LOG_INFO, "Starting security manager %d", hdev);
 
 	if ((dev = hci_open_dev(hdev)) < 0) {
