@@ -1,30 +1,37 @@
-/* 
-	BlueZ - Bluetooth protocol stack for Linux
-	Copyright (C) 2000-2001 Qualcomm Incorporated
-
-	Written 2000,2001 by Maxim Krasnyansky <maxk@qualcomm.com>
-
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License version 2 as
-	published by the Free Software Foundation;
-
-	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
-	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
-	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
-	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
-	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
-	USE OR PERFORMANCE OF THIS SOFTWARE.
-
-	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
-	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
-*/
-
 /*
- * $Id: hciconfig.c,v 1.35 2003/11/18 14:36:31 holtmann Exp $
+ *
+ *  BlueZ - Bluetooth protocol stack for Linux
+ *
+ *  Copyright (C) 2000-2001  Qualcomm Incorporated
+ *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
+ *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation;
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
+ *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
+ *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
+ *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
+ *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
+ *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
+ *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
+ *  SOFTWARE IS DISCLAIMED.
+ *
+ *
+ *  $Id: hciconfig.c,v 1.42 2004/04/28 10:39:47 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -32,6 +39,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <ctype.h>
 #include <getopt.h>
 
 #include <termios.h>
@@ -108,7 +116,7 @@ void print_dev_features(struct hci_dev_info *di, int format)
 			di->features[2], di->features[3],
 			di->features[4], di->features[5],
 			di->features[6], di->features[7],
-			lmp_featurestostr(di->features, "\t\t", 3));
+			lmp_featurestostr(di->features, "\t\t", 63));
 	}
 }
 
@@ -369,11 +377,15 @@ void cmd_name(int ctl, int hdev, char *opt)
 		}
 	} else {
 		char name[248];
+		int i;
 		if (hci_read_local_name(s, sizeof(name), name, 1000) < 0) {
 			printf("Can't read local name on hci%d. %s(%d)\n", 
 				hdev, strerror(errno), errno);
 			exit(1);
 		}
+		for (i = 0; i < 248 && name[i]; i++)
+			if (!isprint(name[i]))
+				name[i] = '.';
 		print_dev_hdr(&di);
 		printf("\tName: '%s'\n", name);
 	}
@@ -640,6 +652,59 @@ void cmd_version(int ctl, int hdev, char *opt)
 		bt_compidtostr(ver.manufacturer), ver.manufacturer);
 }
 
+void cmd_inq_mode(int ctl, int hdev, char *opt)
+{
+	struct hci_request rq;
+	int dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	memset(&rq, 0, sizeof(rq));
+
+	if (opt) {
+		write_inquiry_mode_cp cp;
+
+		cp.mode = atoi(opt);
+
+		rq.ogf = OGF_HOST_CTL;
+		rq.ocf = OCF_WRITE_INQUIRY_MODE;
+		rq.cparam = &cp;
+		rq.clen = WRITE_INQUIRY_MODE_RP_SIZE;
+
+		if (hci_send_req(dd, &rq, 1000) < 0) {
+			printf("Can't set inquiry mode on hci%d. %s(%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+	} else {
+		read_inquiry_mode_rp rp;
+
+		rq.ogf = OGF_HOST_CTL;
+		rq.ocf = OCF_READ_INQUIRY_MODE;
+		rq.rparam = &rp;
+		rq.rlen = READ_INQUIRY_MODE_RP_SIZE;
+
+		if (hci_send_req(dd, &rq, 1000) < 0) {
+			printf("Can't read inquiry mode on hci%d. %s(%d)\n", 
+							hdev, strerror(errno), errno);
+			exit(1);
+		}
+		if (rp.status) {
+			printf("Read inquiry mode on hci%d returned status %d\n", 
+							hdev, rp.status);
+			exit(1);
+		}
+
+		print_dev_hdr(&di);
+		printf("\tInquiry mode: %s\n",
+			rp.mode == 1 ? "Inquiry with RSSI" : "Standard Inquiry");
+	}
+}
+
 void cmd_inq_parms(int ctl, int hdev, char *opt)
 {
 	struct hci_request rq;
@@ -836,6 +901,58 @@ void cmd_page_to(int ctl, int hdev, char *opt)
 	}
 }
 
+void cmd_afh_mode(int ctl, int hdev, char *opt)
+{
+	struct hci_request rq;
+	int dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		printf("Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	memset(&rq, 0, sizeof(rq));
+
+	if (opt) {
+		write_afh_mode_cp cp;
+
+		cp.mode = atoi(opt);
+
+		rq.ogf = OGF_HOST_CTL;
+		rq.ocf = OCF_WRITE_AFH_MODE;
+		rq.cparam = &cp;
+		rq.clen = WRITE_AFH_MODE_RP_SIZE;
+
+		if (hci_send_req(dd, &rq, 1000) < 0) {
+			printf("Can't set AFH mode on hci%d. %s(%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+	} else {
+		read_afh_mode_rp rp;
+
+		rq.ogf = OGF_HOST_CTL;
+		rq.ocf = OCF_READ_AFH_MODE;
+		rq.rparam = &rp;
+		rq.rlen = READ_AFH_MODE_RP_SIZE;
+
+		if (hci_send_req(dd, &rq, 1000) < 0) {
+			printf("Can't read AFH mode on hci%d. %s(%d)\n", 
+							hdev, strerror(errno), errno);
+			exit(1);
+		}
+		if (rp.status) {
+			printf("Read AFH mode on hci%d returned status %d\n", 
+							hdev, rp.status);
+			exit(1);
+		}
+
+		print_dev_hdr(&di);
+		printf("\tAFH mode: %s\n", rp.mode == 1 ? "Enabled" : "Disabled");
+	}
+}
+
 static void print_rev_ericsson(int dd)
 {
 	struct hci_request rq;
@@ -881,6 +998,11 @@ static void print_rev_csr(int dd, uint16_t rev)
 		printf("\tSCO mapping:  %s\n", mapsco ? "PCM" : "HCI");
 }
 
+static void print_rev_broadcom(uint16_t hci_rev, uint16_t lmp_subver)
+{
+	printf("\tFirmware %d.%d.%03d\n", hci_rev, lmp_subver >> 8, lmp_subver & 0xff);
+}
+
 static void print_rev_avm(uint16_t rev)
 {
 	printf("\tFirmware 03.%d.%d\n", rev >> 8, rev & 0xff);
@@ -912,6 +1034,9 @@ static void cmd_revision(int ctl, int hdev, char *opt)
 	case 10:
 		print_rev_csr(dd, ver.hci_rev);
 		break;
+	case 15:
+		print_rev_broadcom(ver.hci_rev, ver.lmp_subver);
+		break;
 	case 31:
 		print_rev_avm(ver.hci_rev);
 		break;
@@ -994,9 +1119,11 @@ struct {
 	{ "class",	cmd_class,	"[class]",	"Get/Set class of device" },
 	{ "voice",	cmd_voice,	"[voice]",	"Get/Set voice setting" },
 	{ "iac",	cmd_iac,	"[iac]",	"Get/Set inquiry access code" },
+	{ "inqmode",	cmd_inq_mode,	"[mode]",	"Get/set inquiry mode" },
 	{ "inqparms",	cmd_inq_parms,	"[win:int]",	"Get/Set inquiry scan window and interval" },
 	{ "pageparms",	cmd_page_parms,	"[win:int]",	"Get/Set page scan window and interval" },
 	{ "pageto",	cmd_page_to,	"[to]",		"Get/Set page timeout" },
+	{ "afhmode",	cmd_afh_mode,	"[mode]",	"Get/Set AFH mode" },
 	{ "aclmtu",	cmd_aclmtu,	"<mtu:pkt>",	"Set ACL MTU and number of packets" },
 	{ "scomtu",	cmd_scomtu,	"<mtu:pkt>",	"Set SCO MTU and number of packets" },
 	{ "features",	cmd_features,	0,		"Display device features" },
