@@ -1,30 +1,37 @@
-/* 
-	BlueZ - Bluetooth protocol stack for Linux
-	Copyright (C) 2000-2001 Qualcomm Incorporated
-
-	Written 2000,2001 by Maxim Krasnyansky <maxk@qualcomm.com>
-
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License version 2 as
-	published by the Free Software Foundation;
-
-	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
-	IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
-	OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
-	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
-	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
-	USE OR PERFORMANCE OF THIS SOFTWARE.
-
-	ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
-	TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
-*/
-
 /*
- * $Id: hcitool.c,v 1.34 2003/12/16 18:45:07 holtmann Exp $
+ *
+ *  BlueZ - Bluetooth protocol stack for Linux
+ *
+ *  Copyright (C) 2000-2001  Qualcomm Incorporated
+ *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
+ *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation;
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
+ *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
+ *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
+ *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
+ *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
+ *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
+ *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
+ *  SOFTWARE IS DISCLAIMED.
+ *
+ *
+ *  $Id: hcitool.c,v 1.43 2004/04/28 10:39:47 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -146,7 +153,7 @@ static void hex_dump(char *pref, int width, unsigned char *buf, int len)
 /* Display local devices */
 
 static struct option dev_options[] = {
-	{"help",    0,0, 'h'},
+	{"help",	0,0, 'h'},
 	{0, 0, 0, 0}
 };
 
@@ -265,7 +272,7 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 		case 'l':
 			length = atoi(optarg);
 			break;
-		
+
 		case 'n':
 			num_rsp = atoi(optarg);
 			break;
@@ -317,7 +324,7 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 /* Remote name */
 
 static struct option name_options[] = {
-	{"help",    0,0, 'h'},
+	{"help",	0,0, 'h'},
 	{0, 0, 0, 0}
 };
 
@@ -410,10 +417,8 @@ static void cmd_info(int dev_id, int argc, char **argv)
 	if (dev_id < 0)
 		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
 
-	if (dev_id < 0) {
+	if (dev_id < 0)
 		dev_id = hci_get_route(&bdaddr);
-		cc = 1;
-	}
 
 	if (dev_id < 0) {
 		fprintf(stderr, "Device is not available or not connected.\n");
@@ -428,26 +433,24 @@ static void cmd_info(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	if (cc) {
-		if (hci_create_connection(dd, &bdaddr, HCI_DM1 | HCI_DH1, 0, 0, &handle, 25000) < 0) {
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr) {
+		perror("Can't get connection info");
+		close(dd);
+		exit(1);
+	}
+
+	bacpy(&cr->bdaddr, &bdaddr);
+	cr->type = ACL_LINK;
+	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+		if (hci_create_connection(dd, &bdaddr, htobs(HCI_DM1 | HCI_DH1), 0, 0, &handle, 25000) < 0) {
 			perror("Can't create connection");
 			close(dd);
 			exit(1);
 		}
-	} else {
-		cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-		if (!cr)
-			return;
-
-		bacpy(&cr->bdaddr, &bdaddr);
-		cr->type = ACL_LINK;
-		if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
-			perror("Get connection info failed");
-			exit(1);
-		}
-
-		handle = cr->conn_info->handle;
-	}
+		cc = 1;
+	} else
+		handle = htobs(cr->conn_info->handle);
 
 	printf("\tBD Address:  %s\n", argv[0]);
 
@@ -465,7 +468,7 @@ static void cmd_info(int dev_id, int argc, char **argv)
 		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n%s\n",
 				features[0], features[1], features[2], features[3],
 				features[4], features[5], features[6], features[7],
-				lmp_featurestostr(features, "\t\t", 3));
+				lmp_featurestostr(features, "\t\t", 63));
 	}
 
 	if (cc)
@@ -657,7 +660,7 @@ static void cmd_cc(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	if (hci_create_connection(dd, &bdaddr, ptype, 0, role, &handle, 1000) < 0)
+	if (hci_create_connection(dd, &bdaddr, htobs(ptype), 0, role, &handle, 1000) < 0)
 		perror("Can't create connection");
 	hci_close_dev(dd);
 }
@@ -703,7 +706,7 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -721,7 +724,7 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	if (hci_disconnect(dd, cr->conn_info->handle, HCI_OE_USER_ENDED_CONNECTION, 100) < 0)
+	if (hci_disconnect(dd, htobs(cr->conn_info->handle), HCI_OE_USER_ENDED_CONNECTION, 100) < 0)
 		perror("Disconnect failed");
 
 	close(dd);
@@ -827,6 +830,7 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 	struct hci_request rq;
 	read_rssi_rp rp;
 	bdaddr_t bdaddr;
+	uint16_t handle;
 	int opt, dd;
 
 	for_each_opt(opt, rssi_options, NULL) {
@@ -853,7 +857,7 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -870,11 +874,13 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 		perror("Get connection info failed");
 		exit(1);
 	}
-	
+
+	handle = htobs(cr->conn_info->handle);
+
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_STATUS_PARAM;
 	rq.ocf    = OCF_READ_RSSI;
-	rq.cparam = &cr->conn_info->handle;
+	rq.cparam = &handle;
 	rq.clen   = 2;
 	rq.rparam = &rp;
 	rq.rlen   = READ_RSSI_RP_SIZE;
@@ -911,6 +917,7 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 	struct hci_request rq;
 	get_link_quality_rp rp;
 	bdaddr_t bdaddr;
+	uint16_t handle;
 	int opt, dd;
 
 	for_each_opt(opt, lq_options, NULL) {
@@ -937,7 +944,7 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -955,14 +962,16 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
+	handle = htobs(cr->conn_info->handle);
+
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_STATUS_PARAM;
 	rq.ocf    = OCF_GET_LINK_QUALITY;
-	rq.cparam = &cr->conn_info->handle;
+	rq.cparam = &handle;
 	rq.clen   = 2;
 	rq.rparam = &rp;
 	rq.rlen   = GET_LINK_QUALITY_RP_SIZE;
-	
+
 	if (hci_send_req(dd, &rq, 100) < 0) {
 		perror("HCI get_link_quality request failed");
 		exit(1);
@@ -1024,7 +1033,7 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -1041,7 +1050,7 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 		perror("Get connection info failed");
 		exit(1);
 	}
-	cp.handle = cr->conn_info->handle;
+	cp.handle = htobs(cr->conn_info->handle);
 
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_HOST_CTL;
@@ -1050,7 +1059,7 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 	rq.clen   = READ_TRANSMIT_POWER_LEVEL_CP_SIZE;
 	rq.rparam = &rp;
 	rq.rlen   = READ_TRANSMIT_POWER_LEVEL_RP_SIZE;
-	
+
 	if (hci_send_req(dd, &rq, 100) < 0) {
 		perror("HCI read transmit power level request failed");
 		exit(1);
@@ -1068,6 +1077,102 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 	free(cr);
 }
 
+/* Get AFH channel map */
+
+static struct option afh_options[] = {
+	{"help",	0,0, 'h'},
+	{0, 0, 0, 0}
+};
+
+static char *afh_help = 
+	"Usage:\n"
+	"\tafh <bdaddr>\n";
+
+static void cmd_afh(int dev_id, int argc, char **argv)
+{
+	struct hci_conn_info_req *cr;
+	struct hci_request rq;
+	read_afh_map_rp rp;
+	bdaddr_t bdaddr;
+	uint16_t handle;
+	int opt, dd;
+
+	for_each_opt(opt, afh_options, NULL) {
+		switch(opt) {
+		default:
+			printf(afh_help);
+			return;
+		}
+	}
+	argc -= optind;
+	argv += optind;
+
+	if (argc < 1) {
+		printf(afh_help);
+		return;
+	}
+
+	str2ba(argv[0], &bdaddr);
+
+	if (dev_id < 0) {
+		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
+		if (dev_id < 0) {
+			fprintf(stderr, "Not connected.\n");
+			exit(1);
+		}
+	}
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		perror("HCI device open failed");
+		exit(1);
+	}
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return;
+
+	bacpy(&cr->bdaddr, &bdaddr);
+	cr->type = ACL_LINK;
+	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+		perror("Get connection info failed");
+		exit(1);
+	}
+
+	handle = htobs(cr->conn_info->handle);
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_STATUS_PARAM;
+	rq.ocf    = OCF_READ_AFH_MAP;
+	rq.cparam = &handle;
+	rq.clen   = 2;
+	rq.rparam = &rp;
+	rq.rlen   = READ_AFH_MAP_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, 100) < 0) {
+		perror("HCI read AFH map request failed");
+		exit(1);
+	}
+
+	if (rp.status) {
+		fprintf(stderr, "HCI read_afh_map cmd failed (0x%2.2X)\n", 
+				rp.status);
+		exit(1);
+	}
+
+	if (rp.mode == 0x01) {
+		int i;
+		printf("AFH map: 0x");
+		for (i = 0; i < 10; i++)
+			printf("%02x", rp.map[i]);
+		printf("\n");
+	} else
+		printf("AFH disabled\n");
+
+	close(dd);
+	free(cr);
+}
+
 /* Set connection packet type */
 
 static struct option cpt_options[] = {
@@ -1113,7 +1218,7 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -1131,7 +1236,7 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	cp.handle   = cr->conn_info->handle;
+	cp.handle   = htobs(cr->conn_info->handle);
 	cp.pkt_type = ptype;
 
 	memset(&rq, 0, sizeof(rq));
@@ -1170,6 +1275,7 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 	read_link_supervision_timeout_rp rp;
 	write_link_supervision_timeout_cp cp;
 	bdaddr_t bdaddr;
+	uint16_t handle;
 	int opt, dd;
 
 	for_each_opt(opt, lst_options, NULL) {
@@ -1196,7 +1302,7 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 			exit(1);
 		}
 	}
- 
+
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		perror("HCI device open failed");
@@ -1214,11 +1320,13 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
+	handle = htobs(cr->conn_info->handle);
+
 	if (argc == 1) {
 		memset(&rq, 0, sizeof(rq));
 		rq.ogf    = OGF_HOST_CTL;
 		rq.ocf    = OCF_READ_LINK_SUPERVISION_TIMEOUT;
-		rq.cparam = &cr->conn_info->handle;
+		rq.cparam = &handle;
 		rq.clen   = 2;
 		rq.rparam = &rp;
 		rq.rlen   = READ_LINK_SUPERVISION_TIMEOUT_RP_SIZE;
@@ -1238,9 +1346,8 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 				rp.link_sup_to, (float)rp.link_sup_to * 0.625);
 		else
 			printf("Link supervision timeout never expires\n");
-	}
-	else {
-		cp.handle      = cr->conn_info->handle;
+	} else {
+		cp.handle      = htobs(cr->conn_info->handle);
 		cp.link_sup_to = strtol(argv[1], NULL, 10);
 
 		memset(&rq, 0, sizeof(rq));
@@ -1259,7 +1366,172 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 	free(cr);
 }
 
-struct {
+/* Request authentication */
+
+static struct option auth_options[] = {
+	{"help",	0,0, 'h'},
+	{0, 0, 0, 0}
+};
+
+static char *auth_help = 
+	"Usage:\n"
+	"\tauth <bdaddr>\n";
+
+static void cmd_auth(int dev_id, int argc, char **argv)
+{
+	struct hci_conn_info_req *cr;
+	struct hci_request rq;
+	auth_requested_cp cp;
+	evt_auth_complete rp;
+	bdaddr_t bdaddr;
+	int opt, dd;
+
+	for_each_opt(opt, auth_options, NULL) {
+		switch(opt) {
+		default:
+			printf(auth_help);
+			return;
+		}
+	}
+	argc -= optind;
+	argv += optind;
+
+	if (argc < 1) {
+		printf(auth_help);
+		return;
+	}
+
+	str2ba(argv[0], &bdaddr);
+
+	if (dev_id < 0) {
+		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
+		if (dev_id < 0) {
+			fprintf(stderr, "Not connected.\n");
+			exit(1);
+		}
+	}
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		perror("HCI device open failed");
+		exit(1);
+	}
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return;
+
+	bacpy(&cr->bdaddr, &bdaddr);
+	cr->type = ACL_LINK;
+	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+		perror("Get connection info failed");
+		exit(1);
+	}
+
+	cp.handle = htobs(cr->conn_info->handle);
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_LINK_CTL;
+	rq.ocf    = OCF_AUTH_REQUESTED;
+	rq.cparam = &cp;
+	rq.clen   = AUTH_REQUESTED_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = EVT_AUTH_COMPLETE_SIZE;
+	rq.event  = EVT_AUTH_COMPLETE;
+
+	if (hci_send_req(dd, &rq, 25000) < 0) {
+		perror("HCI authentication request failed");
+		exit(1);
+	}
+
+	close(dd);
+	free(cr);
+}
+
+/* Activate encryption */
+
+static struct option enc_options[] = {
+	{"help",	0,0, 'h'},
+	{0, 0, 0, 0}
+};
+
+static char *enc_help = 
+	"Usage:\n"
+	"\tenc <bdaddr> [encrypt enable]\n";
+
+static void cmd_enc(int dev_id, int argc, char **argv)
+{
+	struct hci_conn_info_req *cr;
+	struct hci_request rq;
+	set_conn_encrypt_cp cp;
+	evt_encrypt_change rp;
+	bdaddr_t bdaddr;
+	int opt, dd;
+
+	for_each_opt(opt, enc_options, NULL) {
+		switch(opt) {
+		default:
+			printf(enc_help);
+			return;
+		}
+	}
+	argc -= optind;
+	argv += optind;
+
+	if (argc < 1) {
+		printf(enc_help);
+		return;
+	}
+
+	str2ba(argv[0], &bdaddr);
+
+	if (dev_id < 0) {
+		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
+		if (dev_id < 0) {
+			fprintf(stderr, "Not connected.\n");
+			exit(1);
+		}
+	}
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		perror("HCI device open failed");
+		exit(1);
+	}
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return;
+
+	bacpy(&cr->bdaddr, &bdaddr);
+	cr->type = ACL_LINK;
+	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+		perror("Get connection info failed");
+		exit(1);
+	}
+
+	cp.handle = htobs(cr->conn_info->handle);
+	cp.encrypt = (argc > 1) ? atoi(argv[1]) : 1;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_LINK_CTL;
+	rq.ocf    = OCF_SET_CONN_ENCRYPT;
+	rq.cparam = &cp;
+	rq.clen   = SET_CONN_ENCRYPT_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = EVT_ENCRYPT_CHANGE_SIZE;
+	rq.event  = EVT_ENCRYPT_CHANGE;
+
+	if (hci_send_req(dd, &rq, 25000) < 0) {
+		perror("HCI set encryption request failed");
+		exit(1);
+	}
+
+	close(dd);
+	free(cr);
+}
+
+static struct {
 	char *cmd;
 	void (*func)(int dev_id, int argc, char **argv);
 	char *doc;
@@ -1272,14 +1544,17 @@ struct {
 	{ "cmd",  cmd_cmd,  "Submit arbitrary HCI commands"        },
 	{ "con",  cmd_con,  "Display active connections"           },
 	{ "cc",   cmd_cc,   "Create connection to remote device"   },
-	{ "dc",	  cmd_dc,   "Disconnect from remote device"        },
+	{ "dc",   cmd_dc,   "Disconnect from remote device"        },
 	{ "sr",   cmd_sr,   "Switch master/slave role"             },
 	{ "cpt",  cmd_cpt,  "Change connection packet type"        },
 	{ "rssi", cmd_rssi, "Display connection RSSI"              },
 	{ "lq",   cmd_lq,   "Display link quality"                 },
 	{ "tpl",  cmd_tpl,  "Display transmit power level"         },
+	{ "afh",  cmd_afh,  "Display AFH channel map"              },
 	{ "lst",  cmd_lst,  "Set/display link supervision timeout" },
-	{ NULL, NULL, 0}
+	{ "auth", cmd_auth, "Request authentication"               },
+	{ "enc",  cmd_enc,  "Set connection encryption"            },
+	{ NULL, NULL, 0 }
 };
 
 static void usage(void)
